/*
 * Copyright (C) 2011 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by Alex Launi <alex.launi@canonical.com>
 *
 */

using GLib;

namespace Unity.MusicLens
{
  public abstract class SimpleScope : Object
  {
    public Unity.Scope scope { get; protected set; }

    protected abstract int num_results_without_search { get; }
    protected abstract int num_results_global_search { get; }
    protected abstract int num_results_lens_search { get; }

    protected abstract void perform_search (LensSearch? search, Dee.Model results_model, List<FilterParser> filters, int max_results = -1);
    
    public SimpleScope ()
    {
    }

    protected void initialize ()
    {
      /* Listen for filter changes */
      scope.filters_changed.connect (() => {
        scope.queue_search_changed (SearchType.DEFAULT);
      });

      /* No need to search if only the whitespace changes */
      scope.generate_search_key.connect ((lens_search) => {
        return lens_search.search_string.strip ();
      });

      /* Listen for changes to the lens search entry */
      scope.search_changed.connect ((search, search_type, cancellable) => {
        if (search_type == SearchType.DEFAULT)
          update_search_async.begin (search);
        else
          update_global_search_async.begin (search);
      });
    }

    /* based on the filters(?) get the default results to show when the music lens is
     * just opened without filters, like when someone hits Super+M
     */
    private async void update_without_search_async (LensSearch search)
    {
      yield perform_search_async (search, num_results_without_search);
    }
    
    /**
     * results for a global search from just hitting Super. here we want to return
     * a smaller number of results with 0 filters.
     */
    private async void update_global_search_async (LensSearch search)
    {
      if (search_is_invalid (search))
      {
        search.results_model.clear ();
        search.finished ();
        return;
      }

      yield perform_search_async (search, num_results_global_search);
    }
    
    private async void update_search_async (LensSearch search)
    {
      // just pretend like there's no search
      if (search_is_invalid (search))
	{
	  yield update_without_search_async (search);
	  return;
	}
      
      yield perform_search_async (search, num_results_lens_search);
    }

    private async void perform_search_async (LensSearch search, int max_results)
    {
      var results_model = search.results_model;

      List<FilterParser> filters = new List<FilterParser> ();
      Filter filter = scope.get_filter ("genre");
      if (filter.filtering)
	filters.append (new GenreFilterParser (filter as CheckOptionFilter));

      filter = scope.get_filter ("decade");
      if (filter.filtering)
	filters.append (new DecadeFilterParser (filter as MultiRangeFilter));
      
      results_model.clear ();
      perform_search (search, results_model, filters, max_results);
      
      search.finished ();
    }

    protected bool search_is_invalid (LensSearch? search)
    {
      /* This boolean expression is unfolded as we seem to get
       * some null dereference if we join them in a big || expression */
      if (search == null)
        return true;
      else if (search.search_string == null)
        return true;
      
      return search.search_string.strip() == "";
    }
  }
}
